@@ -28,12 +28,16 @@ my @attrs = qw/
     record_class
     attributes
     case_sensitive
+    private
+    protected
+    encode_on_select
     /;
 
 # these actually live in the attributes hash
 my @handy_attrs = qw/
     container
     label hints render_as
+    documentation
     valid_values
     available_values
     autocompleted
@@ -56,7 +60,7 @@ for my $attr (@handy_attrs) {
 
 =head1 NAME
 
-Jifty::DBI::Column
+Jifty::DBI::Column - Encapsulate's a single column in a Jifty::DBI::Record table
 
 =head1 DESCRIPTION
 
@@ -143,6 +147,8 @@ known attributes are:
 
 =item     autocompleted
 
+=item     documentation
+
 =back
 
 =cut
