@@ -19,8 +19,7 @@ if ( my $pattern = $ENV{JIFTY_DBQUERY_CALLER} ) {
         'Jifty::DBI::Handle::simple_query',
         pre => sub {
             return unless $_[1] =~ m/$pattern/;
-            warn $_[1] . '   ' . CORE::join( ',', @_[ 2 .. $#_ ] ) . "\n";
-            Carp::cluck;
+            Carp::cluck($_[1] . '   ' . CORE::join( ',', @_[ 2 .. $#_ ] ));
         }
     );
 }
@@ -88,6 +87,7 @@ sub connect {
         user       => undef,
         password   => undef,
         requiressl => undef,
+        extra      => {},
         @_
     );
 
@@ -109,7 +109,7 @@ sub connect {
     if ( ( !$self->dbh ) || ( !$self->dbh->ping ) || ( $self->dsn ne $dsn ) )
     {
         my $handle
-            = DBI->connect( $self->dsn, $args{'user'}, $args{'password'} )
+            = DBI->connect( $self->dsn, $args{'user'}, $args{'password'}, $args{'extra'} )
             || Carp::croak "Connect Failed $DBI::errstr\n";
 
 #databases do case conversion on the name of columns returned.
@@ -196,6 +196,7 @@ sub build_dsn {
 
     delete $args{'user'};
     delete $args{'password'};
+    delete $args{'extra'};
 
     $self->{'dsn'} = "dbi:$driver:"
         . CORE::join( ';',
@@ -291,14 +292,14 @@ sub _log_sql_statement {
     my $duration  = shift;
     my @bind      = @_;
 
-    my $results = {};
+    my %results;
+    my @log = (Time::HiRes::time(), $statement, [@bind], $duration, \%results);
+
     while (my ($name, $code) = each %{ $self->{'_logsqlhooks'} || {} }) {
-        $results->{$name} = $code->();
+        $results{$name} = $code->(@log);
     }
 
-    push @{ $self->{'StatementLog'} },
-        ( [ Time::HiRes::time(), $statement, [@bind], $duration, $results ] );
-
+    push @{ $self->{'StatementLog'} }, \@log;
 }
 
 =head2 clear_sql_statement_log
@@ -550,6 +551,13 @@ sub simple_query {
     {
         no warnings 'uninitialized';    # undef in bind_values makes DBI sad
         eval { $executed = $sth->execute(@bind_values) };
+
+        # try to ping and reconnect, if the DB connection failed
+        if ($@ and !$self->dbh->ping) {
+            $self->dbh(undef); # don't try pinging again, just connect
+            $self->connect; 
+            eval { $executed = $sth->execute(@bind_values) };
+        }
     }
     if ( $self->log_sql_statements ) {
         $self->_log_sql_statement( $query_string,
@@ -724,12 +732,18 @@ Emulates nested transactions, by keeping a transaction stack depth.
 
 sub begin_transaction {
     my $self = shift;
-    $TRANSDEPTH++;
-    if ( $TRANSDEPTH > 1 ) {
-        return ($TRANSDEPTH);
-    } else {
-        return ( $self->dbh->begin_work );
+
+    if ( $TRANSDEPTH > 0 ) {
+        # We're inside a transaction.
+        $TRANSDEPTH++;
+        return $TRANSDEPTH;
     }
+
+    my $rv = $self->dbh->begin_work;
+    if ($rv) {
+        $TRANSDEPTH++;
+    }
+    return $rv;
 }
 
 =head2 commit
@@ -745,13 +759,18 @@ sub commit {
         Carp::confess(
             "Attempted to commit a transaction with none in progress");
     }
-    $TRANSDEPTH--;
 
-    if ( $TRANSDEPTH == 0 ) {
-        return ( $self->dbh->commit );
-    } else {    #we're inside a transaction
-        return ($TRANSDEPTH);
+    if ($TRANSDEPTH > 1) {
+        # We're inside a nested transaction.
+        $TRANSDEPTH--;
+        return $TRANSDEPTH;
     }
+
+    my $rv = $self->dbh->commit;
+    if ($rv) {
+        $TRANSDEPTH--;
+    }
+    return $rv;
 }
 
 =head2 rollback [FORCE]
@@ -776,16 +795,32 @@ sub rollback {
 #unless ($TRANSDEPTH) {Carp::confess("Attempted to rollback a transaction with none in progress")};
     if ($force) {
         $TRANSDEPTH = 0;
+
+        Jifty::DBI::Record->flush_cache
+            if Jifty::DBI::Record->can('flush_cache');
+
         return ( $dbh->rollback );
     }
 
-    $TRANSDEPTH-- if ( $TRANSDEPTH >= 1 );
-    if ( $TRANSDEPTH == 0 ) {
-        return ( $dbh->rollback );
-    } else {    #we're inside a transaction
-        return ($TRANSDEPTH);
+    if ($TRANSDEPTH == 0) {
+        # We're not actually in a transaction.
+        return 1;
     }
 
+    if ($TRANSDEPTH > 1) {
+        # We're inside a nested transaction.
+        $TRANSDEPTH--;
+        return $TRANSDEPTH;
+    }
+
+    Jifty::DBI::Record->flush_cache
+        if Jifty::DBI::Record->can('flush_cache');
+
+    my $rv = $dbh->rollback;
+    if ($rv) {
+        $TRANSDEPTH--;
+    }
+    return $rv;
 }
 
 =head2 force_rollback
@@ -845,7 +880,6 @@ L<Jifty::DBI::Collection> object'.
 
 This performs the join.
 
-
 =cut
 
 sub join {
@@ -905,7 +939,7 @@ sub join {
                 @args{qw/alias1 alias2/}   = @args{qw/alias2 alias1/};
                 @args{qw/column1 column2/} = @args{qw/column2 column1/};
 
-                return $self->Jifty::DBI::Collection::limit(
+                return $args{'collection'}->limit(
                     entry_aggregator => 'AND',
                     @_,
                     quote_value => 0,
@@ -1074,7 +1108,7 @@ sub may_be_null {
         # left joins on the left side so later we'll get 1 AND x expression
         # which equal to x, so we just skip it
         next if $join->{'type'} eq 'LEFT';
-        next unless $join->{'depends_on'} eq $args{'alias'};
+        next unless $join->{'depends_on'} && ($join->{'depends_on'} eq $args{'alias'});
 
         my @tmp = map { ( '(', @$_, ')', $join->{'entry_aggregator'} ) }
             values %{ $join->{'criteria'} };
@@ -1231,6 +1265,28 @@ sub log {
 
 }
 
+=head2 canonical_true
+
+This returns the canonical true value for this database. For example, in SQLite
+it is 1 but in Postgres it is 't'.
+
+The default is 1.
+
+=cut
+
+sub canonical_true { 1 }
+
+=head2 canonical_false
+
+This returns the canonical false value for this database. For example, in SQLite
+it is 0 but in Postgres it is 'f'.
+
+The default is 0.
+
+=cut
+
+sub canonical_false { 0 }
+
 =head2 DESTROY
 
 When we get rid of the L<Jifty::DBI::Handle>, we need to disconnect
